@@ -11,15 +11,42 @@
 TortoiseHg About dialog - PyQt4 version
 """
 
+from __future__ import absolute_import
+
 import sys
 
-from tortoisehg.hgqt import qtlib
-from tortoisehg.util import version, hglib, paths
-from tortoisehg.util.i18n import _
+from .qtcore import (
+    PYQT_VERSION_STR,
+    QSettings,
+    QSize,
+    QT_VERSION_STR,
+    QTimer,
+    QUrl,
+    Qt,
+    pyqtSlot,
+)
+from .qtgui import (
+    QDialog,
+    QDialogButtonBox,
+    QFont,
+    QLabel,
+    QLayout,
+    QPixmap,
+    QPlainTextEdit,
+    QVBoxLayout,
+)
+from .qtnetwork import (
+    QNetworkAccessManager,
+    QNetworkRequest,
+)
 
-from PyQt4.QtCore import *
-from PyQt4.QtGui import *
-from PyQt4.QtNetwork import QNetworkAccessManager, QNetworkRequest
+from ..util import (
+    hglib,
+    paths,
+    version,
+)
+from ..util.i18n import _
+from . import qtlib
 
 class AboutDialog(QDialog):
     """Dialog for showing info about TortoiseHg"""
@@ -52,7 +79,7 @@ class AboutDialog(QDialog):
         self.copyright_lbl = QLabel()
         self.copyright_lbl.setAlignment(Qt.AlignCenter)
         self.copyright_lbl.setText('\n'
-                + _('Copyright 2008-2017 Steve Borho and others'))
+                + _('Copyright 2008-2019 Steve Borho and others'))
         self.vbox.addWidget(self.copyright_lbl)
         self.courtesy_lbl = QLabel()
         self.courtesy_lbl.setAlignment(Qt.AlignCenter)
@@ -89,7 +116,7 @@ class AboutDialog(QDialog):
         self._readsettings()
 
         # Spawn it later, so that the dialog gets visible quickly.
-        QTimer.singleShot(0, self.getUpdateInfo)
+        QTimer.singleShot(1000, self.getUpdateInfo)
         self._newverreply = None
 
     def getVersionInfo(self):
@@ -142,9 +169,7 @@ class AboutDialog(QDialog):
         except (IndexError, ImportError, ValueError):
             pass
         try:
-            thgv = version.version()
-            if '+' in thgv:
-                thgv = thgv[:thgv.index('+')]
+            thgv = version.package_version()
             curver = tuple([int(p) for p in thgv.split('.')])
         except ValueError:
             curver = (0,0,0)
@@ -166,7 +191,7 @@ class AboutDialog(QDialog):
 
     def _readsettings(self):
         s = QSettings()
-        self.restoreGeometry(s.value('about/geom').toByteArray())
+        self.restoreGeometry(qtlib.readByteArray(s, 'about/geom'))
 
     def _writesettings(self):
         s = QSettings()
@@ -211,7 +236,7 @@ class LicenseDialog(QDialog):
 
     def _readsettings(self):
         s = QSettings()
-        self.restoreGeometry(s.value('license/geom').toByteArray())
+        self.restoreGeometry(qtlib.readByteArray(s, 'license/geom'))
 
     def _writesettings(self):
         s = QSettings()
