@@ -5,21 +5,61 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2, incorporated herein by reference.
 
-import os, re, signal, struct, sys, time
+from __future__ import absolute_import
 
-from PyQt4.QtCore import QIODevice, QObject, QProcess, QTimer
-from PyQt4.QtCore import pyqtSignal, pyqtSlot
+import os
+import signal
+import struct
+import sys
+import time
 
-from mercurial import util
+from .qtcore import (
+    QBuffer,
+    QIODevice,
+    QObject,
+    QProcess,
+    QProcessEnvironment,
+    QTimer,
+    pyqtSignal,
+    pyqtSlot,
+)
 
-from tortoisehg.util import hglib, paths, pipeui
-from tortoisehg.hgqt.i18n import _
-from tortoisehg.hgqt import thread
+from ..util import (
+    hglib,
+    paths,
+    pipeui,
+)
+from ..util.i18n import _
 
-# TODO: should be moved from thread once CmdThread is superseded by CmdServer
-ProgressMessage = thread.ProgressMessage
+class ProgressMessage(tuple):
+    __slots__ = ()
 
-class UiHandler(QObject):
+    def __new__(cls, topic, pos, item='', unit='', total=None):
+        return tuple.__new__(cls, (topic, pos, item, unit, total))
+
+    @property
+    def topic(self):
+        return self[0]  # unicode
+    @property
+    def pos(self):
+        return self[1]  # int or None
+    @property
+    def item(self):
+        return self[2]  # unicode
+    @property
+    def unit(self):
+        return self[3]  # unicode
+    @property
+    def total(self):
+        return self[4]  # int or None
+
+    def __repr__(self):
+        names = ('topic', 'pos', 'item', 'unit', 'total')
+        fields = ('%s=%r' % (n, v) for n, v in zip(names, self))
+        return '%s(%s)' % (self.__class__.__name__, ', '.join(fields))
+
+
+class UiHandler(object):
     """Interface to handle user interaction of Mercurial commands"""
 
     NoInput = 0
@@ -27,6 +67,10 @@ class UiHandler(QObject):
     PasswordInput = 2
     ChoiceInput = 3
 
+    def __init__(self):
+        self._datain = None
+        self._dataout = None
+
     def setPrompt(self, text, mode, default=None):
         pass
 
@@ -34,6 +78,30 @@ class UiHandler(QObject):
         # '' to use default; None to abort
         return ''
 
+    def setDataInputDevice(self, device):
+        # QIODevice to read data from; None to disable data input
+        self._datain = device
+
+    def setDataOutputDevice(self, device):
+        # QIODevice to write data output; None to disable capturing
+        self._dataout = device
+
+    def inputAtEnd(self):
+        if not self._datain:
+            return True
+        return self._datain.atEnd()
+
+    def readInput(self, size):
+        # b'' for EOF; None for error (per PyQt's QIODevice.read() convention)
+        if not self._datain:
+            return b''
+        return self._datain.read(size)
+
+    def writeOutput(self, data, label):
+        if not self._dataout or label.startswith('ui.') or ' ui.' in label:
+            return -1
+        return self._dataout.write(data)
+
 
 def _createDefaultUiHandler(uiparent):
     if uiparent is None:
@@ -45,7 +113,11 @@ def _createDefaultUiHandler(uiparent):
     # in place of
     #   cmdagent.runCommand(..., self)
     from tortoisehg.hgqt import cmdui
-    return cmdui.InteractiveUiHandler(None, uiparent)
+    return cmdui.InteractiveUiHandler(uiparent)
+
+
+class _ProtocolError(Exception):
+    """Error while processing server message; must be caught by CmdWorker"""
 
 
 class CmdWorker(QObject):
@@ -63,7 +135,7 @@ class CmdWorker(QObject):
 
     serviceStateChanged = pyqtSignal(int)
     commandFinished = pyqtSignal(int)
-    outputReceived = pyqtSignal(unicode, unicode)
+    outputReceived = pyqtSignal(str, str)
     progressReceived = pyqtSignal(ProgressMessage)
 
     def serviceState(self):
@@ -83,9 +155,13 @@ class CmdWorker(QObject):
 
 
 _localprocexts = [
+    'tortoisehg.util.hgcommands',
     'tortoisehg.util.partialcommit',
     'tortoisehg.util.pipeui',
     ]
+_localserverexts = [
+    'tortoisehg.util.hgdispatch',
+    ]
 
 if os.name == 'nt':
     # to translate WM_CLOSE posted by QProcess.terminate()
@@ -99,37 +175,53 @@ else:
         os.kill(int(proc.pid()), signal.SIGINT)
 
 def _fixprocenv(proc):
-    env = os.environ.copy()
-    if not getattr(sys, 'frozen', False):
+    env = QProcessEnvironment.systemEnvironment()
+    # disable flags and extensions that might break our output parsing
+    # (e.g. "defaults" arguments, "PAGER" of "email --test")
+    env.insert('HGPLAINEXCEPT', 'alias,i18n,revsetalias')
+    # since sys.path may contain the script directory which the hg process
+    # wouldn't see, we have to filter it out
+    libpaths = set(sys.path)
+    libpaths.discard(os.path.dirname(os.path.realpath(sys.argv[0])))
+    thgroot = paths.get_prog_root()
+    if not getattr(sys, 'frozen', False) and thgroot not in libpaths:
         # make sure hg process can look up our modules
-        env['PYTHONPATH'] = (paths.get_prog_root() + os.pathsep
-                             + env.get('PYTHONPATH', ''))
-    # not using setProcessEnvironment() for compatibility with PyQt 4.6
-    proc.setEnvironment([hglib.tounicode('%s=%s' % p) for p in env.iteritems()])
+        pypath = hglib.tounicode(thgroot)
+        if env.contains('PYTHONPATH'):
+            pypath += os.pathsep + unicode(env.value('PYTHONPATH'))
+        env.insert('PYTHONPATH', pypath)
+    proc.setProcessEnvironment(env)
 
-def _proccmdline(exts):
-    cmdline = map(hglib.tounicode, paths.get_hg_command())
-    for e in exts:
-        cmdline.extend(('--config', 'extensions.%s=' % e))
-    return cmdline
+def _proccmdline(ui, exts):
+    configs = [(section, name, value)
+               for section, name, value in ui.walkconfig()
+               if ui.configsource(section, name) == '--config']
+    configs.extend(('extensions', e, '') for e in exts)
+    cmdline = list(paths.get_hg_command())
+    for section, name, value in configs:
+        cmdline.extend(('--config', '%s.%s=%s' % (section, name, value)))
+    return map(hglib.tounicode, cmdline)
 
 
 class CmdProc(CmdWorker):
     'Run mercurial command in separate process'
 
-    def __init__(self, parent=None, cwd=None):
+    def __init__(self, ui, parent=None, cwd=None):
         super(CmdProc, self).__init__(parent)
+        self._ui = ui
+        self._uihandler = None
         self._proc = proc = QProcess(self)
         _fixprocenv(proc)
         if cwd:
             proc.setWorkingDirectory(cwd)
-        proc.finished.connect(self.commandFinished)
+        proc.finished.connect(self._finish)
         proc.readyReadStandardOutput.connect(self._stdout)
         proc.readyReadStandardError.connect(self._stderr)
         proc.error.connect(self._handleerror)
 
     def startCommand(self, cmdline, uihandler):
-        fullcmdline = _proccmdline(_localprocexts)
+        self._uihandler = uihandler
+        fullcmdline = _proccmdline(self._ui, _localprocexts)
         fullcmdline.extend(cmdline)
         self._proc.start(fullcmdline[0], fullcmdline[1:], QIODevice.ReadOnly)
 
@@ -141,19 +233,27 @@ class CmdProc(CmdWorker):
     def isCommandRunning(self):
         return self._proc.state() != QProcess.NotRunning
 
+    @pyqtSlot(int)
+    def _finish(self, ret):
+        self._uihandler = None
+        self.commandFinished.emit(ret)
+
+    @pyqtSlot(QProcess.ProcessError)
     def _handleerror(self, error):
         if error == QProcess.FailedToStart:
             self.outputReceived.emit(_('failed to start command\n'),
                                      'ui.error')
-            self.commandFinished.emit(-1)
+            self._finish(-1)
         elif error != QProcess.Crashed:
             self.outputReceived.emit(_('error while running command\n'),
                                      'ui.error')
 
+    @pyqtSlot()
     def _stdout(self):
         data = self._proc.readAllStandardOutput().data()
         self._processRead(data, '')
 
+    @pyqtSlot()
     def _stderr(self):
         data = self._proc.readAllStandardError().data()
         self._processRead(data, 'ui.error')
@@ -161,7 +261,11 @@ class CmdProc(CmdWorker):
     def _processRead(self, fulldata, defaultlabel):
         for data in pipeui.splitmsgs(fulldata):
             msg, label = pipeui.unpackmsg(data)
+            if (not defaultlabel  # only stdout
+                and self._uihandler.writeOutput(msg, label) >= 0):
+                continue
             msg = hglib.tounicode(msg)
+            label = hglib.tounicode(label)
             if 'ui.progress' in label.split():
                 progress = ProgressMessage(*pipeui.unpackprogress(msg))
                 self.progressReceived.emit(progress)
@@ -172,16 +276,21 @@ class CmdProc(CmdWorker):
 class CmdServer(CmdWorker):
     """Run Mercurial commands in command server process"""
 
-    def __init__(self, parent=None, cwd=None):
+    def __init__(self, ui, parent=None, cwd=None):
         super(CmdServer, self).__init__(parent)
-        self._uihandler = None
+        self._ui = ui
+        self._uihandler = UiHandler()
         self._readchtable = self._idlechtable
         self._readq = []  # (ch, data or datasize), ...
         # deadline for arrival of hello message and immature data
-        self._readtimer = QTimer(self, interval=5000, singleShot=True)
+        sec = ui.configint('tortoisehg', 'cmdserver.readtimeout', 30)
+        self._readtimer = QTimer(self, interval=sec * 1000, singleShot=True)
         self._readtimer.timeout.connect(self._onReadTimeout)
+        self._proc = self._createProc(cwd)
+        self._servicestate = CmdWorker.NotRunning
 
-        self._proc = proc = QProcess(self)
+    def _createProc(self, cwd):
+        proc = QProcess(self)
         _fixprocenv(proc)
         if cwd:
             proc.setWorkingDirectory(cwd)
@@ -190,7 +299,7 @@ class CmdServer(CmdWorker):
         proc.setReadChannel(QProcess.StandardOutput)
         proc.readyRead.connect(self._onReadyRead)
         proc.readyReadStandardError.connect(self._onReadyReadError)
-        self._servicestate = CmdWorker.NotRunning
+        return proc
 
     def serviceState(self):
         return self._servicestate
@@ -208,7 +317,14 @@ class CmdServer(CmdWorker):
             self._changeServiceState(CmdWorker.Restarting)
 
     def _startService(self):
-        cmdline = _proccmdline(_localprocexts)
+        if self._proc.bytesToWrite() > 0:
+            # QTBUG-44517: recreate QProcess to discard remainder of last
+            # request; otherwise it would be written to new process
+            oldproc = self._proc
+            self._proc = self._createProc(oldproc.workingDirectory())
+            oldproc.setParent(None)
+
+        cmdline = _proccmdline(self._ui, _localprocexts + _localserverexts)
         cmdline.extend(['serve', '--cmdserver', 'pipe',
                         '--config', 'ui.interactive=True'])
         self._readchtable = self._hellochtable
@@ -239,7 +355,7 @@ class CmdServer(CmdWorker):
 
     @pyqtSlot()
     def _onServiceFinished(self):
-        self._uihandler = None
+        self._uihandler = UiHandler()
         self._readchtable = self._idlechtable
         del self._readq[:]
         self._readtimer.stop()
@@ -257,7 +373,12 @@ class CmdServer(CmdWorker):
     def startCommand(self, cmdline, uihandler):
         assert self._servicestate == CmdWorker.Ready
         assert not self.isCommandRunning()
-        data = hglib.fromunicode('\0'.join(cmdline))
+        try:
+            data = hglib.fromunicode('\0'.join(cmdline))
+        except UnicodeEncodeError, inst:
+            self._emitError(_('failed to encode command: %s') % inst)
+            self._finishCommand(-1)
+            return
         self._uihandler = uihandler
         self._readchtable = self._runcommandchtable
         self._proc.write('runcommand\n')
@@ -269,7 +390,7 @@ class CmdServer(CmdWorker):
         _interruptproc(self._proc)
 
     def _finishCommand(self, ret):
-        self._uihandler = None
+        self._uihandler = UiHandler()
         self._readchtable = self._idlechtable
         self.commandFinished.emit(ret)
 
@@ -329,11 +450,12 @@ class CmdServer(CmdWorker):
                 except KeyError:
                     if not ch.isupper():
                         continue
-                    self._emitError(_('unexpected response on required '
-                                      'channel %r') % ch)
-                    self.stopService()
-                    return
-                chfunc(self, dataorsize)
+                    raise _ProtocolError(_('unexpected response on required '
+                                           'channel %r') % ch)
+                chfunc(self, ch, dataorsize)
+        except _ProtocolError, inst:
+            self._emitError(inst.args[0])
+            self.stopService()
         except Exception:
             self.stopService()
             raise
@@ -344,26 +466,26 @@ class CmdServer(CmdWorker):
         for data in pipeui.splitmsgs(fulldata):
             msg, label = pipeui.unpackmsg(data)
             msg = hglib.tounicode(msg)
+            label = hglib.tounicode(label)
             self.outputReceived.emit(msg, label or 'ui.error')
 
-    def _processHello(self, data):
+    def _processHello(self, _ch, data):
         try:
             fields = dict(l.split(':', 1) for l in data.splitlines())
             capabilities = fields['capabilities'].split()
         except (KeyError, ValueError):
-            self._emitError(_('invalid "hello" message: %r') % data)
-            self.stopService()
-            return
+            raise _ProtocolError(_('invalid "hello" message: %r') % data)
         if 'runcommand' not in capabilities:
-            self._emitError(_('no "runcommand" capability'))
-            self.stopService()
-            return
+            raise _ProtocolError(_('no "runcommand" capability'))
         self._readchtable = self._idlechtable
         self._changeServiceState(CmdWorker.Ready)
 
-    def _processOutput(self, data):
+    def _processOutput(self, ch, data):
         msg, label = pipeui.unpackmsg(data)
+        if ch == 'o' and self._uihandler.writeOutput(msg, label) >= 0:
+            return
         msg = hglib.tounicode(msg)
+        label = hglib.tounicode(label)
         labelset = label.split()
         if 'ui.progress' in labelset:
             progress = ProgressMessage(*pipeui.unpackprogress(msg))
@@ -380,23 +502,37 @@ class CmdServer(CmdWorker):
         else:
             self.outputReceived.emit(msg, label)
 
-    def _processCommandResult(self, data):
+    def _processCommandResult(self, _ch, data):
         try:
             ret, = struct.unpack('>i', data)
-            # command server returns raw result code in hg<3.0 (5d4606bec54c)
-            ret &= 255
         except struct.error:
-            self._emitError(_('corrupted command result: %r') % data)
-            self.stopService()
-            return
+            raise _ProtocolError(_('corrupted command result: %r') % data)
         self._finishCommand(ret)
 
-    def _processLineRequest(self, size):
+    def _processInputRequest(self, _ch, size):
+        data = self._uihandler.readInput(size)
+        if data is None:
+            self._emitError(_('error while reading from data input'))
+            self._writeBlock(b'')
+            return
+        if not data and not self._uihandler.inputAtEnd():
+            # TODO: should stop processing until readyRead()
+            self._emitError(_('asynchronous read is not implement yet'))
+        self._writeBlock(data)
+
+    def _processLineRequest(self, _ch, size):
+        # TODO: if no prompt observed, this should be redirected to
+        # self._uihandler.readLineInput(size + 1)
         text = self._uihandler.getLineInput()
         if text is None:
             self._writeBlock('')
             return
-        data = hglib.fromunicode(text) + '\n'
+        try:
+            data = hglib.fromunicode(text) + '\n'
+        except UnicodeEncodeError, inst:
+            self._emitError(_('failed to encode input: %s') % inst)
+            self.abortCommand()
+            return
         for start in xrange(0, len(data), size):
             self._writeBlock(data[start:start + size])
 
@@ -414,56 +550,27 @@ class CmdServer(CmdWorker):
         'o': _processOutput,
         'e': _processOutput,
         'r': _processCommandResult,
-        # implement 'I' (data input) channel if necessary
+        'I': _processInputRequest,
         'L': _processLineRequest,
         }
 
 
 _workertypes = {
-    'thread': thread.CmdThread,
     'proc': CmdProc,
     'server': CmdServer,
     }
 
 
-_urlpassre = re.compile(r'^([a-zA-Z0-9+.\-]+://[^:@/]*):[^@/]+@')
-
-def _reprcmdarg(arg):
-    arg = _urlpassre.sub(r'\1:***@', arg)
-    arg = arg.replace('\n', '^M')
-
-    # only for display; no use to construct command string for os.system()
-    if not arg or ' ' in arg or '\\' in arg or '"' in arg:
-        return '"%s"' % arg.replace('"', '\\"')
-    else:
-        return arg
-
-def _prettifycmdline(cmdline):
-    r"""Build pretty command-line string for display
-
-    >>> _prettifycmdline(['log', 'foo\\bar', '', 'foo bar', 'foo"bar'])
-    'log "foo\\bar" "" "foo bar" "foo\\"bar"'
-    >>> _prettifycmdline(['log', '--template', '{node}\n'])
-    'log --template {node}^M'
-
-    mask password in url-like string:
-
-    >>> _prettifycmdline(['push', 'http://foo123:bar456@example.org/'])
-    'push http://foo123:***@example.org/'
-    >>> _prettifycmdline(['clone', 'svn+http://:bar@example.org:8080/trunk/'])
-    'clone svn+http://:***@example.org:8080/trunk/'
-    """
-    return ' '.join(_reprcmdarg(e) for e in cmdline)
-
 class CmdSession(QObject):
     """Run Mercurial commands in a background thread or process"""
 
     commandFinished = pyqtSignal(int)
     # in order to receive only notification messages of session state, use
     # "controlMessage"; otherwise use "outputReceived"
-    controlMessage = pyqtSignal(unicode)
-    outputReceived = pyqtSignal(unicode, unicode)
+    controlMessage = pyqtSignal(str)
+    outputReceived = pyqtSignal(str, str)
     progressReceived = pyqtSignal(ProgressMessage)
+    readyRead = pyqtSignal()
 
     def __init__(self, cmdlines, uihandler, parent=None):
         super(CmdSession, self).__init__(parent)
@@ -474,13 +581,12 @@ class CmdSession(QObject):
         self._abortbyuser = False
         self._erroroutputs = []
         self._warningoutputs = []
+        self._dataoutrbuf = QBuffer(self)
         self._exitcode = 0
         if not cmdlines:
             # assumes null session is failure for convenience
             self._exitcode = -1
 
-    ### Public Methods ###
-
     def run(self, worker):
         '''Execute Mercurial command'''
         if self._worker or self._qnextp >= len(self._queue):
@@ -510,9 +616,6 @@ class CmdSession(QObject):
 
     def isRunning(self):
         """True if a command is running; False if finished or not started yet"""
-        # keep "running" until just before emitting commandFinished. if worker
-        # is QThread, isRunning() is cleared earlier than _onCommandFinished,
-        # because inter-thread signal is queued.
         return bool(self._worker) and self._qnextp > 0
 
     def errorString(self):
@@ -530,7 +633,51 @@ class CmdSession(QObject):
         """Integer return code of the last command"""
         return self._exitcode
 
-    ### Private Method ###
+    def setCaptureOutput(self, enabled):
+        """If enabled, data outputs (without "ui.*" label) are queued and
+        outputReceived signal is not emitted in that case.  This is useful
+        for receiving data to be parsed or copied to the clipboard.
+        """
+        # pseudo FIFO between client "rbuf" and worker "wbuf"; not efficient
+        # for large data since all outputs will be stored in memory
+        if enabled:
+            self._dataoutrbuf.open(QIODevice.ReadOnly | QIODevice.Truncate)
+            dataoutwbuf = QBuffer(self._dataoutrbuf.buffer())
+            dataoutwbuf.bytesWritten.connect(self.readyRead)
+            dataoutwbuf.open(QIODevice.WriteOnly)
+        else:
+            self._dataoutrbuf.close()
+            dataoutwbuf = None
+        self.setOutputDevice(dataoutwbuf)
+
+    def setInputDevice(self, device):
+        """If set, data will be read from the specified device on data input
+        request"""
+        if self.isRunning():
+            raise RuntimeError('command already running')
+        self._uihandler.setDataInputDevice(device)
+
+    def setOutputDevice(self, device):
+        """If set, data outputs will be sent to the specified device"""
+        if self.isRunning():
+            raise RuntimeError('command already running')
+        self._uihandler.setDataOutputDevice(device)
+
+    def read(self, maxlen):
+        """Read output if capturing enabled; ui messages are not included"""
+        return self._dataoutrbuf.read(maxlen)
+
+    def readAll(self):
+        return self._dataoutrbuf.readAll()
+
+    def readLine(self, maxlen=0):
+        return self._dataoutrbuf.readLine(maxlen)
+
+    def canReadLine(self):
+        return self._dataoutrbuf.canReadLine()
+
+    def peek(self, maxlen):
+        return self._dataoutrbuf.peek(maxlen)
 
     def _connectWorker(self, worker):
         self._worker = worker
@@ -558,12 +705,8 @@ class CmdSession(QObject):
     def _runNext(self):
         cmdline = self._queue[self._qnextp]
         self._qnextp += 1
-
-        self._abortbyuser = False
-        del self._erroroutputs[:]
-        del self._warningoutputs[:]
+        self._emitControlMessage('% hg ' + hglib.prettifycmdline(cmdline))
         self._worker.startCommand(cmdline, self._uihandler)
-        self._emitControlMessage('% hg ' + _prettifycmdline(cmdline))
 
     def _finish(self, ret):
         self._qnextp = len(self._queue)
@@ -571,8 +714,6 @@ class CmdSession(QObject):
         self._exitcode = ret
         self.commandFinished.emit(ret)
 
-    ### Signal Handlers ###
-
     @pyqtSlot(int)
     def _onWorkerStateChanged(self, state):
         if state == CmdWorker.Ready:
@@ -600,7 +741,7 @@ class CmdSession(QObject):
         else:
             self._runNext()
 
-    @pyqtSlot(unicode, unicode)
+    @pyqtSlot(str, str)
     def _captureOutput(self, msg, label):
         if not label:
             return  # fast path
@@ -624,6 +765,11 @@ def nullCmdSession():
     >>> sess.abort()  # should not change flags
     >>> sess.isFinished(), sess.isRunning(), sess.isAborted(), sess.exitCode()
     (True, False, False, -1)
+
+    Null session can be set up just like one made by runCommand().  It can be
+    used as an object representing failure or canceled operation.
+
+    >>> sess.setOutputDevice(QBuffer())
     """
     return CmdSession([], UiHandler())
 
@@ -642,39 +788,28 @@ class CmdAgent(QObject):
     # Inactive session is not started by the agent, so agent.commandFinished
     # won't be emitted when waiting session is aborted.
     commandFinished = pyqtSignal(CmdSession)
-    outputReceived = pyqtSignal(unicode, unicode)
+    outputReceived = pyqtSignal(str, str)
     progressReceived = pyqtSignal(ProgressMessage)
 
     # isBusy() is False when the last commandFinished is emitted, but you
     # shouldn't rely on the emission order of busyChanged and commandFinished.
 
-    def __init__(self, ui, parent=None):
+    def __init__(self, ui, parent=None, cwd=None, worker=None):
         super(CmdAgent, self).__init__(parent)
         self._ui = ui
-        self._cwd = None
-        self._workers = {}
+        self._worker = self._createWorker(cwd, worker or 'server')
         self._sessqueue = []  # [active, waiting...]
         self._runlater = QTimer(self, interval=0, singleShot=True)
         self._runlater.timeout.connect(self._runNextSession)
 
-    def workingDirectory(self):
-        return self._cwd
-
-    def setWorkingDirectory(self, cwd):
-        if self._workers:
-            raise RuntimeError('cannot change working directory after run')
-        self._cwd = unicode(cwd)
-
     def isServiceRunning(self):
         stoppedstates = (CmdWorker.NoService, CmdWorker.NotRunning)
-        return util.any(worker.serviceState() not in stoppedstates
-                        for worker in self._workers.itervalues())
+        return self._worker.serviceState() not in stoppedstates
 
     def stopService(self):
         """Shut down back-end services so that this can be deleted safely or
         reconfigured; serviceStopped will be emitted asynchronously"""
-        for worker in self._workers.itervalues():
-            worker.stopService()
+        self._worker.stopService()
 
     @pyqtSlot()
     def _tryEmitServiceStopped(self):
@@ -684,8 +819,8 @@ class CmdAgent(QObject):
     def isBusy(self):
         return bool(self._sessqueue)
 
-    def _enqueueSession(self, sess, workername):
-        self._sessqueue.append((sess, workername))
+    def _enqueueSession(self, sess):
+        self._sessqueue.append(sess)
         if len(self._sessqueue) == 1:
             self.busyChanged.emit(self.isBusy())
             # make sure no command signals emitted in the current context
@@ -702,17 +837,17 @@ class CmdAgent(QObject):
 
     def _cleanupWaitingSession(self):
         for i in reversed(xrange(1, len(self._sessqueue))):
-            sess, _workername = self._sessqueue[i]
+            sess = self._sessqueue[i]
             if sess.isFinished():
                 del self._sessqueue[i]
                 sess.setParent(None)
 
-    def runCommand(self, cmdline, uihandler=None, worker=None):
+    def runCommand(self, cmdline, uihandler=None):
         """Executes a single Mercurial command asynchronously and returns
         new CmdSession object"""
-        return self.runCommandSequence([cmdline], uihandler, worker=worker)
+        return self.runCommandSequence([cmdline], uihandler)
 
-    def runCommandSequence(self, cmdlines, uihandler=None, worker=None):
+    def runCommandSequence(self, cmdlines, uihandler=None):
         """Executes a series of Mercurial commands asynchronously and returns
         new CmdSession object which will provide notification signals.
 
@@ -721,46 +856,33 @@ class CmdAgent(QObject):
         as the parent widget of the default InteractiveUiHandler.  If uihandler
         is None, no interactive prompt will be displayed.
 
+        If the specified uihandler is a UiHandler object, it should be created
+        per request in order to avoid sharing the same uihandler across several
+        CmdSession objects.
+
         CmdSession object will be disowned on command finished.  The specified
         uihandler is unrelated to the lifetime of CmdSession object.
 
         If one of the preceding command exits with non-zero status, the
         following commands won't be executed.
         """
-        if worker and worker not in _workertypes:
-            raise ValueError('invalid worker type: %r' % worker)
-        if not worker:
-            worker = self._defaultWorkerName()
         if not isinstance(uihandler, UiHandler):
             uihandler = _createDefaultUiHandler(uihandler)
         sess = CmdSession(cmdlines, uihandler, self)
         sess.commandFinished.connect(self._onCommandFinished)
         sess.controlMessage.connect(self._forwardControlMessage)
-        self._enqueueSession(sess, worker)
+        self._enqueueSession(sess)
         return sess
 
     def abortCommands(self):
         """Abort running and queued commands; all command sessions will emit
         commandFinished"""
-        for sess, _workername in self._sessqueue[:]:
+        for sess in self._sessqueue[:]:
             sess.abort()
 
-    def _defaultWorkerName(self):
-        # hidden config for debugging or testing experimental worker
-        name = self._ui.config('tortoisehg', 'cmdworker', 'thread')
-        if name not in _workertypes:
-            self.outputReceived.emit(_("ignoring invalid cmdworker '%s'\n")
-                                     % hglib.tounicode(name), 'ui.warning')
-            return 'thread'
-        return name
-
-    def _prepareWorker(self, name):
-        worker = self._workers.get(name)
-        if worker:
-            assert not worker.isCommandRunning() or thread.HAVE_QTBUG_30251
-            return worker
+    def _createWorker(self, cwd, name):
         self._ui.debug("creating cmdworker '%s'\n" % name)
-        self._workers[name] = worker = _workertypes[name](self, self._cwd)
+        worker = _workertypes[name](self._ui, self, cwd)
         worker.serviceStateChanged.connect(self._tryEmitServiceStopped)
         worker.outputReceived.connect(self.outputReceived)
         worker.progressReceived.connect(self.progressReceived)
@@ -768,15 +890,16 @@ class CmdAgent(QObject):
 
     @pyqtSlot()
     def _runNextSession(self):
-        sess, workername = self._sessqueue[0]
-        worker = self._prepareWorker(workername)
+        sess = self._sessqueue[0]
+        worker = self._worker
+        assert not worker.isCommandRunning()
         sess.run(worker)
         # start after connected to sess so that it can receive immediate error
         worker.startService()
 
     @pyqtSlot()
     def _onCommandFinished(self):
-        sess, _workername = self._sessqueue[0]
+        sess = self._sessqueue[0]
         if not sess.isFinished():
             # waiting session is aborted, just delete it
             self._cleanupWaitingSession()
@@ -785,6 +908,6 @@ class CmdAgent(QObject):
         self.commandFinished.emit(sess)
         sess.setParent(None)
 
-    @pyqtSlot(unicode)
+    @pyqtSlot(str)
     def _forwardControlMessage(self, msg):
         self.outputReceived.emit(msg + '\n', 'control')
