@@ -17,17 +17,58 @@
 Qt4 dialogs to display hg revisions of a file
 """
 
+from __future__ import absolute_import
+
 import difflib
 
-from tortoisehg.util import hglib
-from tortoisehg.hgqt.i18n import _
-from tortoisehg.hgqt import qtlib, repomodel, blockmatcher, lexers
-from tortoisehg.hgqt import filectxactions, fileview, repoview, revpanel
-from tortoisehg.hgqt.qscilib import Scintilla
+from .qsci import (
+    QsciScintilla,
+)
+from .qtcore import (
+    QEvent,
+    QItemSelectionModel,
+    QPoint,
+    QSettings,
+    QTimer,
+    QUrl,
+    Qt,
+    pyqtSlot,
+)
+from .qtgui import (
+    QAbstractItemView,
+    QAction,
+    QColor,
+    QDesktopServices,
+    QFrame,
+    QKeySequence,
+    QHBoxLayout,
+    QMainWindow,
+    QMenu,
+    QPainter,
+    QPen,
+    QSplitter,
+    QToolBar,
+    QVBoxLayout,
+    QWidget,
+)
 
-from PyQt4.QtCore import *
-from PyQt4.QtGui import *
-from PyQt4.Qsci import QsciScintilla
+from mercurial import (
+    scmutil,
+)
+
+from ..util import hglib
+from ..util.i18n import _
+from . import (
+    blockmatcher,
+    filectxactions,
+    fileview,
+    lexers,
+    qtlib,
+    repomodel,
+    repoview,
+    revpanel,
+)
+from .qscilib import Scintilla
 
 sides = ('left', 'right')
 otherside = {'left': 'right', 'right': 'left'}
@@ -52,6 +93,19 @@ def _setupFileMenu(menu, fileactions):
         else:
             menu.addSeparator()
 
+def _fileDataListForSelection(model, selmodel):
+    # since FileRevModel is a model for single file, this creates single
+    # FileData between two revisions instead of a list for each selection
+    indexes = sorted(selmodel.selectedRows())
+    if not indexes:
+        return []
+    if len(indexes) == 1:
+        fd = model.fileData(indexes[0])
+    else:
+        fd = model.fileData(indexes[0], indexes[-1])
+    return [fd]
+
+
 class _FileDiffScintilla(Scintilla):
     def paintEvent(self, event):
         super(_FileDiffScintilla, self).paintEvent(event)
@@ -79,22 +133,48 @@ class _FileDiffScintilla(Scintilla):
             y = line * scale
             p.drawLine(0, y, viewport.width(), y)
 
+# Minimal wrapper to make RepoAgent always returns unfiltered repo.  Because
+# filelog_grapher can't take account of hidden changesets, all child widgets
+# of file dialog need to take unfiltered repo instance.
+#
+# TODO: this should be removed if filelog_grapher (and FileRevModel) are
+# superseded by revset-based implementation.
+class _UnfilteredRepoAgentProxy(object):
+    def __init__(self, repoagent):
+        self._repoagent = repoagent
+
+    def rawRepo(self):
+        repo = self._repoagent.rawRepo()
+        return repo.unfiltered()
+
+    def runCommand(self, cmdline, uihandler=None, overlay=True):
+        cmdline = ['--hidden'] + cmdline
+        return self._repoagent.runCommand(cmdline, uihandler, overlay)
+
+    def runCommandSequence(self, cmdlines, uihandler=None, overlay=True):
+        cmdlines = [['--hidden'] + l for l in cmdlines]
+        return self._repoagent.runCommandSequence(cmdlines, uihandler, overlay)
+
+    def __getattr__(self, name):
+        return getattr(self._repoagent, name)
+
 class _AbstractFileDialog(QMainWindow):
 
     def __init__(self, repoagent, filename):
         QMainWindow.__init__(self)
-        self._repoagent = repoagent
+        self._repoagent = _UnfilteredRepoAgentProxy(repoagent)
 
         self.setupUi()
         self._show_rev = None
 
-        assert not isinstance(filename, (unicode, QString))
+        assert not isinstance(filename, unicode)
         self.filename = filename
 
         self.setWindowTitle(_('Hg file log viewer [%s] - %s')
                             % (repoagent.displayName(),
                                hglib.tounicode(filename)))
         self.setWindowIcon(qtlib.geticon('hg-log'))
+        self.setIconSize(qtlib.toolBarIconSize())
 
         self.createActions()
         self.setupToolbars()
@@ -137,9 +217,9 @@ class FileLogDialog(_AbstractFileDialog):
         s.beginGroup('filelog')
         try:
             self.textView.loadSettings(s, 'fileview')
-            self.restoreGeometry(s.value('geom').toByteArray())
-            self.splitter.restoreState(s.value('splitter').toByteArray())
-            self.revpanel.set_expanded(s.value('revpanel.expanded').toBool())
+            self.restoreGeometry(qtlib.readByteArray(s, 'geom'))
+            self.splitter.restoreState(qtlib.readByteArray(s, 'splitter'))
+            self.revpanel.set_expanded(qtlib.readBool(s, 'revpanel.expanded'))
         finally:
             s.endGroup()
 
@@ -170,13 +250,14 @@ class FileLogDialog(_AbstractFileDialog):
         self.splitter = QSplitter(Qt.Vertical)
         self.setCentralWidget(self.splitter)
         cs = ('fileLogDialog', _('File History Log Columns'))
-        self.repoview = repoview.HgRepoView(self.repo, cs[0], cs, self.splitter)
+        self.repoview = repoview.HgRepoView(self._repoagent, cs[0], cs,
+                                            self.splitter)
 
         self.contentframe = QFrame(self.splitter)
 
         vbox = QVBoxLayout()
         vbox.setSpacing(0)
-        vbox.setMargin(0)
+        vbox.setContentsMargins(0, 0, 0, 0)
         self.contentframe.setLayout(vbox)
 
         self.revpanel = revpanel.RevPanelWidget(self.repo)
@@ -196,8 +277,8 @@ class FileLogDialog(_AbstractFileDialog):
         self.editToolbar.addAction(self.actionForward)
 
     def setupModels(self):
-        self.filerevmodel = repomodel.FileRevModel(self.repo, self.filename,
-                                                   parent=self)
+        self.filerevmodel = repomodel.FileRevModel(
+            self._repoagent, self.filename, parent=self)
         self.repoview.setModel(self.filerevmodel)
         self.repoview.revisionSelected.connect(self.onRevisionSelected)
         self.repoview.revisionActivated.connect(self.onRevisionActivated)
@@ -205,7 +286,7 @@ class FileLogDialog(_AbstractFileDialog):
         selmodel = self.repoview.selectionModel()
         selmodel.selectionChanged.connect(self._onRevisionSelectionChanged)
         self.filerevmodel.showMessage.connect(self.statusBar().showMessage)
-        self.filerevmodel.filled.connect(self.modelFilled)
+        QTimer.singleShot(0, self._updateRepoViewForModel)
 
     def createActions(self):
         self.actionClose.triggered.connect(self.close)
@@ -226,18 +307,25 @@ class FileLogDialog(_AbstractFileDialog):
         self.addActions(self._fileactions.actions())
 
     def _updateFileActions(self):
-        model = self.filerevmodel
         selmodel = self.repoview.selectionModel()
-        curfd = model.fileData(selmodel.currentIndex())
-        selfds = map(model.fileData, selmodel.selectedRows())
-        self._fileactions.setFileData(curfd, selfds)
+        selfds = _fileDataListForSelection(self.filerevmodel, selmodel)
+        self._fileactions.setFileDataList(selfds)
+        if len(selmodel.selectedRows()) > 1:
+            texts = {'visualDiff': _('Diff Selected &Changesets'),
+                     'visualDiffFile': _('&Diff Selected File Revisions')}
+        else:
+            texts = {'visualDiff': _('Diff &Changeset to Parent'),
+                     'visualDiffFile': _('&Diff to Parent')}
+        for n, t in texts.iteritems():
+            self._fileactions.action(n).setText(t)
 
     @pyqtSlot()
     def _updateHistoryActions(self):
         self.actionBack.setEnabled(self.repoview.canGoBack())
         self.actionForward.setEnabled(self.repoview.canGoForward())
 
-    def modelFilled(self):
+    @pyqtSlot()
+    def _updateRepoViewForModel(self):
         self.repoview.resizeColumns()
         if self._show_rev is not None:
             index = self.filerevmodel.indexLinkedFromRev(self._show_rev)
@@ -246,8 +334,7 @@ class FileLogDialog(_AbstractFileDialog):
             return  # already set by goto()
         else:
             index = self.filerevmodel.index(0,0)
-        if index is not None:
-            self.repoview.setCurrentIndex(index)
+        self.repoview.setCurrentIndex(index)
 
     @pyqtSlot(QPoint, object)
     def viewMenuRequest(self, point, selection):
@@ -256,7 +343,7 @@ class FileLogDialog(_AbstractFileDialog):
             return
         menu = QMenu(self)
         if len(selection) == 2:
-            for name in ['visualDiffRevs', 'visualDiffFileRevs']:
+            for name in ['visualDiff', 'visualDiffFile']:
                 menu.addAction(self._fileactions.action(name))
         else:
             _setupFileMenu(menu, self._fileactions)
@@ -277,13 +364,14 @@ class FileLogDialog(_AbstractFileDialog):
         self.revdetails.show()
         self.revdetails.raise_()
 
-    @pyqtSlot(QString)
+    @pyqtSlot(str)
     def onLinkActivated(self, link):
         link = unicode(link)
         if ':' in link:
             scheme, param = link.split(':', 1)
             if scheme == 'cset':
-                rev = self.repo[hglib.fromunicode(param)].rev()
+                rev = scmutil.revsymbol(self.repo,
+                                        hglib.fromunicode(param)).rev()
                 return self.goto(rev)
         QDesktopServices.openUrl(QUrl(link))
 
@@ -294,7 +382,6 @@ class FileLogDialog(_AbstractFileDialog):
         self.textView.verticalScrollBar().setValue(pos)
         self.revpanel.set_revision(rev)
         self.revpanel.update(repo = self.repo)
-        self._updateFileActions()
 
     @pyqtSlot()
     def _onRevisionSelectionChanged(self):
@@ -314,7 +401,7 @@ class FileLogDialog(_AbstractFileDialog):
 
     def goto(self, rev):
         index = self.filerevmodel.indexLinkedFromRev(rev)
-        if index is not None:
+        if index.isValid():
             self.repoview.setCurrentIndex(index)
         else:
             self._show_rev = rev
@@ -348,8 +435,8 @@ class FileDiffDialog(_AbstractFileDialog):
         s = QSettings()
         s.beginGroup('filediff')
         try:
-            self.restoreGeometry(s.value('geom').toByteArray())
-            self.splitter.restoreState(s.value('splitter').toByteArray())
+            self.restoreGeometry(qtlib.readByteArray(s, 'geom'))
+            self.splitter.restoreState(qtlib.readByteArray(s, 'splitter'))
         finally:
             s.endGroup()
 
@@ -387,7 +474,7 @@ class FileDiffDialog(_AbstractFileDialog):
         self._repoViews = []
         cs = ('fileDiffDialogLeft', _('File Differences Log Columns'))
         for cfgname in [cs[0], 'fileDiffDialogRight']:
-            w = repoview.HgRepoView(self.repo, cfgname, cs, self)
+            w = repoview.HgRepoView(self._repoagent, cfgname, cs, self)
             w.setSelectionMode(QAbstractItemView.SingleSelection)
             self.horizontalLayout.addWidget(w)
             self._repoViews.append(w)
@@ -493,12 +580,14 @@ class FileDiffDialog(_AbstractFileDialog):
     def setupModels(self):
         self.filedata = {'left': None, 'right': None}
         self._invbarchanged = False
-        self.filerevmodel = repomodel.FileRevModel(self.repo, self.filename,
-                                                   parent=self)
-        self.filerevmodel.filled.connect(self.modelFilled)
+        self.filerevmodel = repomodel.FileRevModel(
+            self._repoagent, self.filename, parent=self)
         for w in self._repoViews:
             w.setModel(self.filerevmodel)
             w.menuRequested.connect(self.viewMenuRequest)
+            selmodel = w.selectionModel()
+            selmodel.selectionChanged.connect(self._onRevisionSelectionChanged)
+        QTimer.singleShot(0, self._updateRepoViewForModel)
 
     def createActions(self):
         self.actionClose.triggered.connect(self.close)
@@ -521,17 +610,17 @@ class FileDiffDialog(_AbstractFileDialog):
         self._fileactions = filectxactions.FilectxActions(self._repoagent, self)
         self.addActions(self._fileactions.actions())
 
-    def _updateFileActionsForView(self, view):
-        model = self.filerevmodel
-        curfd = model.fileData(view.currentIndex())
-        self._fileactions.setFileData(curfd)
+    def _updateFileActionsForSelection(self, selmodel):
+        selfds = _fileDataListForSelection(self.filerevmodel, selmodel)
+        self._fileactions.setFileDataList(selfds)
 
     def setupToolbars(self):
         self.editToolbar.addSeparator()
         self.editToolbar.addAction(self.actionNextDiff)
         self.editToolbar.addAction(self.actionPrevDiff)
 
-    def modelFilled(self):
+    @pyqtSlot()
+    def _updateRepoViewForModel(self):
         for w in self._repoViews:
             w.resizeColumns()
         if self._show_rev is not None:
@@ -551,7 +640,7 @@ class FileDiffDialog(_AbstractFileDialog):
             return False
         elif watched in self._repoViews:
             if event.type() == QEvent.FocusIn:
-                self._updateFileActionsForView(watched)
+                self._updateFileActionsForSelection(watched.selectionModel())
             return False
         else:
             return super(FileDiffDialog, self).eventFilter(watched, event)
@@ -564,15 +653,19 @@ class FileDiffDialog(_AbstractFileDialog):
         else:
             side = 'left'
         path = self.filerevmodel.graph.nodesdict[rev].extra[0]
-        fc = self.repo.changectx(rev).filectx(path)
+        fc = self.repo[rev].filectx(path)
         data = hglib.tounicode(fc.data())
         self.filedata[side] = data.splitlines()
         self.update_diff(keeppos=otherside[side])
-        self._updateFileActionsForView(self.sender())
+
+    @pyqtSlot()
+    def _onRevisionSelectionChanged(self):
+        assert isinstance(self.sender(), QItemSelectionModel)
+        self._updateFileActionsForSelection(self.sender())
 
     def goto(self, rev):
         index = self.filerevmodel.indexLinkedFromRev(rev)
-        if index is not None:
+        if index.isValid():
             if index.row() == 0:
                 index = self.filerevmodel.index(1, 0)
             self._repoViews[0].setCurrentIndex(index)
@@ -706,7 +799,7 @@ class FileDiffDialog(_AbstractFileDialog):
             self.update_page_steps(keeppos)
             self.timer.start()
 
-    @qtlib.senderSafeSlot()
+    @pyqtSlot()
     def _syncColumnsVisibility(self):
         src = self.sender()
         dest = dict(zip(self._repoViews, reversed(self._repoViews)))[src]
