@@ -8,16 +8,39 @@
 # Foundation; either version 2 of the License, or (at your option) any later
 # version.
 
-import os, re
+from __future__ import absolute_import
 
-from PyQt4.QtCore import *
-from PyQt4.QtGui import *
+import os
+import re
 
-from mercurial import error, subrepo, util
-from mercurial import match as matchmod
+from .qtcore import (
+    QAbstractItemModel,
+    QMimeData,
+    QModelIndex,
+    QUrl,
+    Qt,
+    pyqtSignal,
+    pyqtSlot,
+)
+from .qtgui import (
+    QCompleter,
+    QFileIconProvider,
+    QIcon,
+)
 
-from tortoisehg.util import hglib
-from tortoisehg.hgqt import filedata, qtlib, status, visdiff
+from mercurial import (
+    error,
+    match as matchmod,
+    scmutil,
+)
+
+from ..util import hglib
+from . import (
+    filedata,
+    qtlib,
+    status,
+    visdiff,
+)
 
 _subrepoType2IcoMap = {
     'hg': 'hg',
@@ -53,7 +76,7 @@ class ManifestModel(QAbstractItemModel):
         self._repoagent = repoagent
 
         self._namefilter = unicode(namefilter or '')
-        assert util.all(c in 'MARSC' for c in statusfilter)
+        assert all(c in 'MARSC' for c in statusfilter)
         self._statusfilter = statusfilter
         self._changedfilesonly = False
         self._nodeop = _nodeopmap[bool(flat)]
@@ -141,7 +164,7 @@ class ManifestModel(QAbstractItemModel):
             return ic
         st = status.statusTypes[e.status]
         if st.icon:
-            icOverlay = qtlib.geticon(st.icon[:-4])
+            icOverlay = qtlib.geticon(st.icon)
             ic = qtlib.getoverlaidicon(ic, icOverlay)
 
         return ic
@@ -290,7 +313,7 @@ class ManifestModel(QAbstractItemModel):
         """Return the current name filter"""
         return self._namefilter
 
-    @pyqtSlot(unicode)
+    @pyqtSlot(str)
     def setNameFilter(self, pattern):
         """Filter file name by partial match of glob pattern"""
         pattern = unicode(pattern)
@@ -308,7 +331,7 @@ class ManifestModel(QAbstractItemModel):
     def setStatusFilter(self, status):
         """Filter file tree by change status 'MARSC'"""
         status = str(status)
-        assert util.all(c in 'MARSC' for c in status)
+        assert all(c in 'MARSC' for c in status)
         if self._statusfilter == status:
             return  # for performance reason
         self._statusfilter = status
@@ -351,10 +374,13 @@ class ManifestModel(QAbstractItemModel):
         assert len(self._rootentry) == 0
         newroote = self._rootentry.copyskel()
         self._populateNodes(newroote)
-        self.beginInsertRows(parent, 0, len(newroote) - 1)
+        last = len(newroote) - 1
+        if last >= 0:
+            self.beginInsertRows(parent, 0, last)
         self._rootentry = newroote
         self._rootpopulated = True
-        self.endInsertRows()
+        if last >= 0:
+            self.endInsertRows()
         self.revLoaded.emit(self.rev())
 
     def _repopulateNodes(self, newnodeop=None, newroote=None):
@@ -595,9 +621,9 @@ def _populaterepo(roote, repo, nodeop, statusfilter, match):
 def _comparesubstate(state1, state2):
     if state1 == state2:
         return 'C'
-    elif state1 == subrepo.nullstate:
+    elif state1 == hglib.nullsubrepostate:
         return 'A'
-    elif state2 == subrepo.nullstate:
+    elif state2 == hglib.nullsubrepostate:
         return 'R'
     else:
         return 'M'
@@ -608,8 +634,8 @@ def _populatesubrepos(roote, repo, nodeop, statusfilter, match):
     subpaths = set(pctx.substate)
     subpaths.update(ctx.substate)
     for path in subpaths:
-        substate = ctx.substate.get(path, subrepo.nullstate)
-        psubstate = pctx.substate.get(path, subrepo.nullstate)
+        substate = ctx.substate.get(path, hglib.nullsubrepostate)
+        psubstate = pctx.substate.get(path, hglib.nullsubrepostate)
         e = _Entry()
         e.status = _comparesubstate(psubstate, substate)
         if e.status == 'R':
@@ -622,11 +648,11 @@ def _populatesubrepos(roote, repo, nodeop, statusfilter, match):
         # if git or svn executable not found
         if (nodeop.subreporecursive and e.subkind == 'hg' and e.status != 'R'
             and os.path.isdir(repo.wjoin(path))):
-            smatch = matchmod.narrowmatcher(path, match)
+            smatch = matchmod.subdirmatcher(path, match)
             try:
                 srepo = ctx.sub(path)._repo
-                e.ctx = srepo[substate[1]]
-                e.pctx = srepo[psubstate[1] or 'null']
+                e.ctx = scmutil.revsymbol(srepo, substate[1])
+                e.pctx = scmutil.revsymbol(srepo, psubstate[1] or 'null')
                 _populaterepo(e, srepo, nodeop, statusfilter, smatch)
             except (error.RepoError, EnvironmentError):
                 pass
@@ -655,12 +681,12 @@ class ManifestCompleter(QCompleter):
     def splitPath(self, path):
         """
         >>> c = ManifestCompleter()
-        >>> c.splitPath(QString('foo/bar'))
+        >>> c.splitPath(u'foo/bar')
         [u'foo', u'bar']
 
         trailing slash appends extra '', so that QCompleter can descend to
         next level:
-        >>> c.splitPath(QString('foo/'))
+        >>> c.splitPath(u'foo/')
         [u'foo', u'']
         """
         return unicode(path).split('/')
