@@ -44,9 +44,11 @@ from .qtgui import (
 
 from mercurial import (
     error,
-    patch,
     phases,
-    util,
+    scmutil,
+)
+from mercurial.utils import (
+    procutil,
 )
 
 from ..util import (
@@ -69,6 +71,7 @@ from . import (
     matching,
     merge,
     mq,
+    phabreview,
     postreview,
     prune,
     purge,
@@ -83,6 +86,7 @@ from . import (
     tag,
     thgimport,
     thgstrip,
+    topics,
     update,
     visdiff,
 )
@@ -106,14 +110,17 @@ from .sync import SyncWidget
 # fixed = the changeset is considered permanent
 # applied = an applied patch
 # qgoto = applied patch or qparent
+# isdraftorwd = working directory or changset is draft
+
 _ENABLE_MENU_FUNCS = {
-    'isrev'  : lambda ap, wd, tags: not wd,
-    'iswd'   : lambda ap, wd, tags: bool(wd),
-    'isctx'  : lambda ap, wd, tags: True,
-    'fixed'  : lambda ap, wd, tags: not (ap or wd),
-    'applied': lambda ap, wd, tags: ap,
-    'qgoto'  : lambda ap, wd, tags: ('qparent' in tags) or (ap),
-    'istrue' : lambda ap, wd, tags: True,
+    'isrev'  : lambda ap, wd, tags, ph: not wd,
+    'iswd'   : lambda ap, wd, tags, ph: bool(wd),
+    'isctx'  : lambda ap, wd, tags, ph: True,
+    'fixed'  : lambda ap, wd, tags, ph: not (ap or wd),
+    'applied': lambda ap, wd, tags, ph: ap,
+    'qgoto'  : lambda ap, wd, tags, ph: ('qparent' in tags) or (ap),
+    'istrue' : lambda ap, wd, tags, ph: True,
+    'isdraftorwd': lambda ap, wd, tags, ph: bool(ph > 0 or wd),
 }
 
 
@@ -632,7 +639,7 @@ class RepoWidget(QWidget):
         """Select and scroll to the specified revision"""
         try:
             # try instant look up
-            if hglib.fromunicode(revspec) in self.repo:
+            if scmutil.isrevsymbol(self.repo, hglib.fromunicode(revspec)):
                 self.repoview.goto(revspec)
                 return
         except error.LookupError:
@@ -704,11 +711,9 @@ class RepoWidget(QWidget):
                 if '\0' in earlybytes:
                     continue
                 pf.seek(0)
-                data = patch.extract(self.repo.ui, pf)
-                filename = data.get('filename')
-                if filename:
-                    filepaths.append(p)
-                    os.unlink(filename)
+                with hglib.extractpatch(self.repo.ui, pf) as data:
+                    if data.get('filename'):
+                        filepaths.append(p)
             except EnvironmentError:
                 pass
         return filepaths
@@ -871,7 +876,7 @@ class RepoWidget(QWidget):
     def switchToPreferredTaskTab(self):
         tw = self.taskTabsWidget
         rev = self.rev
-        ctx = self.repo.changectx(rev)
+        ctx = self.repo[rev]
         if rev is None or ('mq' in self.repo.extensions() and 'qtip' in ctx.tags()
                            and self.repo['.'].rev() == rev):
             # Clicking on working copy or on the topmost applied patch
@@ -915,7 +920,7 @@ class RepoWidget(QWidget):
         if isinstance(rev, basestring):
             qgoto = True
         else:
-            ctx = self.repo.changectx(rev)
+            ctx = self.repo[rev]
             if 'qparent' in ctx.tags() or ctx.thgmqappliedpatch():
                 qgoto = True
             if 'qtip' in ctx.tags():
@@ -1092,7 +1097,7 @@ class RepoWidget(QWidget):
         allunapp = False
         if 'mq' in self.repo.extensions():
             for rev in selection:
-                if not self.repo.changectx(rev).thgmqunappliedpatch():
+                if not self.repo[rev].thgmqunappliedpatch():
                     break
             else:
                 allunapp = True
@@ -1106,14 +1111,15 @@ class RepoWidget(QWidget):
             self.multipleSelectionMenu(point, selection)
 
     def _selectionMenuExec(self, point, selection, menu, menuitems):
-        ctxs = [self.repo.changectx(rev) for rev in selection]
+        ctxs = [self.repo[rev] for rev in selection]
         applied = [ctx.thgmqappliedpatch() for ctx in ctxs]
         working = [ctx.rev() is None for ctx in ctxs]
         tags = [ctx.tags() for ctx in ctxs]
+        phases = [ctx.phase() for ctx in ctxs]
 
         for item in menuitems:
             enabled = all(item.enableFunc(*args)
-                          for args in zip(applied, working, tags))
+                          for args in zip(applied, working, tags, phases))
             item.setEnabled(enabled)
 
         menu.exec_(point)
@@ -1254,6 +1260,8 @@ class RepoWidget(QWidget):
               self.tagToRevision)
         entry(items, menu, None, enablefuncs['isrev'], _('Boo&kmark...'),
               'hg-bookmarks', self.bookmarkRevision)
+        entry(items, menu, 'topic', enablefuncs['isdraftorwd'], _('Top&ic...'),
+              'topic', self.topicsRevision)
         entry(items, menu, 'gpg', enablefuncs['fixed'], _('Sig&n...'),
               'hg-sign', self.signRevision)
         entry(items, menu)
@@ -1281,7 +1289,8 @@ class RepoWidget(QWidget):
 
         submenu = menu.addMenu(_('Change &Phase to'))
         submenu.triggered.connect(self._changePhaseByMenu)
-        for pnum, pname in enumerate(phases.phasenames):
+        # TODO: filter out hidden names better
+        for pnum, pname in enumerate(phases.phasenames[:3]):
             a = entry(items, submenu, None, enablefuncs['isrev'], pname)
             a.setData(pnum)
         entry(items, menu)
@@ -1320,6 +1329,10 @@ class RepoWidget(QWidget):
               _('Post to Re&view Board...'), 'reviewboard',
               self.sendToReviewBoard)
 
+        entry(items, menu, 'phabricator', enablefuncs['isrev'],
+              _('Post to Phabricator...'), 'phabricator',
+              self.sendToPhabricator)
+
         entry(items, menu, 'rupdate', enablefuncs['fixed'],
               _('&Remote Update...'), 'hg-update', self.rupdate)
 
@@ -1405,9 +1418,9 @@ class RepoWidget(QWidget):
             dlg.restart(str(revB), str(revA))
         def compressDlg():
             ctxa, ctxb = map(self.repo.hgchangectx, self.menuselection)
-            if ctxa.ancestor(ctxb) == ctxb:
+            if ctxa.ancestor(ctxb).rev() == ctxb.rev():
                 revs = self.menuselection[:]
-            elif ctxa.ancestor(ctxb) == ctxa:
+            elif ctxa.ancestor(ctxb).rev() == ctxa.rev():
                 revs = [self.menuselection[1], self.menuselection[0]]
             else:
                 InfoMsgBox(_('Unable to compress history'),
@@ -1473,6 +1486,12 @@ class RepoWidget(QWidget):
             a.triggered.connect(self.sendToReviewBoard)
             menu.addAction(a)
 
+        if 'phabricator' in self.repo.extensions():
+            menu.addSeparator()
+            a = QAction(_('Post Selected to Phabricator...'), self)
+            a.triggered.connect(self.sendToPhabricator)
+            menu.addAction(a)
+
         self._setupCustomSubmenu(items, menu,
                                  'workbench.pairselection.custom-menu')
 
@@ -1544,6 +1563,10 @@ class RepoWidget(QWidget):
               _('Post Selected to Review Board...'), None,
               self.sendToReviewBoard)
 
+        entry(items, menu, 'phabricator', enablefuncs['istrue'],
+              _('Post Selected to Phabricator...'), None,
+              self.sendToPhabricator)
+
         self._setupCustomSubmenu(items, menu,
                                  'workbench.multipleselection.custom-menu')
 
@@ -1797,13 +1820,27 @@ class RepoWidget(QWidget):
 
     @pyqtSlot()
     def mergeWithRevision(self):
+        # Don't use self.rev (i.e. the current revision.) This is a context
+        # menu handler, and the menu is open for the selected rows, not for
+        # the current row.
+        revisions = self.repoview.selectedRevisions()
+        if len(revisions) != 1:
+            QMessageBox.warning(self, _('Unable to merge'),
+                                _('Please select a revision to merge.'))
+            return
+        rev = revisions[0]
+        if not isinstance(rev, int):
+            QMessageBox.warning(self, _('Unable to merge'),
+                                _('Cannot merge with a pseudo revision %r.')
+                                % rev)
+            return
         pctx = self.repo['.']
-        octx = self.repo[self.rev]
+        octx = self.repo[rev]
         if pctx == octx:
             QMessageBox.warning(self, _('Unable to merge'),
                 _('You cannot merge a revision with itself'))
             return
-        self._dialogs.open(RepoWidget._createMergeDialog, self.rev)
+        self._dialogs.open(RepoWidget._createMergeDialog, rev)
 
     def _createMergeDialog(self, rev):
         return merge.MergeDialog(self._repoagent, rev, self)
@@ -1816,6 +1853,10 @@ class RepoWidget(QWidget):
         dlg = bookmark.BookmarkDialog(self._repoagent, self.rev, self)
         dlg.exec_()
 
+    def topicsRevision(self):
+        dlg = topics.TopicsDialog(self._repoagent, self.rev, self)
+        dlg.exec_()
+
     def signRevision(self):
         dlg = sign.SignDialog(self._repoagent, self.rev, self)
         dlg.exec_()
@@ -1859,6 +1900,14 @@ class RepoWidget(QWidget):
     def _createPostReviewDialog(self, revs):
         return postreview.PostReviewDialog(self.repo.ui, self._repoagent, revs)
 
+    @pyqtSlot()
+    def sendToPhabricator(self):
+        self._dialogs.open(RepoWidget._createPhabReviewDialog,
+                           tuple(self.repoview.selectedRevisions()))
+
+    def _createPhabReviewDialog(self, revs):
+        return phabreview.PhabReviewDialog(self._repoagent, revs)
+
     def rupdate(self):
         import rupdate
         dlg = rupdate.createRemoteUpdateDialog(self._repoagent, self.rev, self)
@@ -2025,7 +2074,7 @@ class RepoWidget(QWidget):
             QMessageBox.warning(self, _('Cannot import selected revision'),
                 _('The selected revision (rev #%d) cannot be imported '
                 'because it is not a descendant of ''qparent'' (rev #%d)') \
-                % (self.rev, self.repo['qparent'].rev()))
+                % (self.rev, scmutil.revsymbol(self.repo, 'qparent').rev()))
             return
 
         patchdir = self.repo.vfs.join('patches')
@@ -2087,7 +2136,7 @@ class RepoWidget(QWidget):
     def qgotoRevision(self, rev):
         """Make REV the top applied patch"""
         mqw = self._mqActions
-        ctx = self.repo.changectx(rev)
+        ctx = self.repo[rev]
         if 'qparent'in ctx.tags():
             mqw.popAllPatches()
         else:
@@ -2096,12 +2145,12 @@ class RepoWidget(QWidget):
     def qrename(self):
         sel = self.menuselection[0]
         if not isinstance(sel, str):
-            sel = self.repo.changectx(sel).thgmqpatchname()
+            sel = self.repo[sel].thgmqpatchname()
         self._mqActions.renamePatch(hglib.tounicode(sel))
 
     def _qpushRevision(self, move=False, exact=False):
         """QPush REV with the selected options"""
-        ctx = self.repo.changectx(self.rev)
+        ctx = self.repo[self.rev]
         patchname = hglib.tounicode(ctx.thgmqpatchname())
         self._mqActions.pushPatch(patchname, move=move, exact=exact)
 
@@ -2133,7 +2182,7 @@ class RepoWidget(QWidget):
 
         # 2. Expand internal workbench variables
         def filelist2str(filelist):
-            return ' '.join(util.shellquote(
+            return ' '.join(procutil.shellquote(
                             os.path.normpath(self.repo.wjoin(filename)))
                             for filename in filelist)
         if files is None:
