@@ -5,12 +5,41 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2, incorporated herein by reference.
 
-from tortoisehg.util import hglib, i18n
-from tortoisehg.util.i18n import _
-from tortoisehg.hgqt import cmdcore, qtlib
+from __future__ import absolute_import
 
-from PyQt4.QtCore import *
-from PyQt4.QtGui import *
+from .qtcore import (
+    Qt,
+    pyqtSlot,
+)
+from .qtgui import (
+    QCheckBox,
+    QComboBox,
+    QDialog,
+    QDialogButtonBox,
+    QFormLayout,
+    QFrame,
+    QHBoxLayout,
+    QLabel,
+    QLayout,
+    QLineEdit,
+    QSizePolicy,
+    QVBoxLayout,
+    QWidget,
+)
+
+from mercurial import (
+    scmutil,
+)
+
+from ..util import (
+    hglib,
+    i18n,
+)
+from ..util.i18n import _
+from . import (
+    cmdcore,
+    qtlib,
+)
 
 keep = i18n.keepgettext()
 
@@ -30,20 +59,17 @@ class TagDialog(QDialog):
         base = QVBoxLayout()
         base.setSpacing(0)
         base.setContentsMargins(0, 0, 0, 0)
-        base.setSizeConstraint(QLayout.SetFixedSize)
+        base.setSizeConstraint(QLayout.SetMinAndMaxSize)
         self.setLayout(base)
 
-        # main layout box
-        box = QVBoxLayout()
-        box.setSpacing(8)
-        box.setContentsMargins(8, 8, 8, 8)
-        self.layout().addLayout(box)
-
+        formwidget = QWidget(self)
+        formwidget.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
         form = QFormLayout(fieldGrowthPolicy=QFormLayout.AllNonFixedFieldsGrow)
-        box.addLayout(form)
+        formwidget.setLayout(form)
+        base.addWidget(formwidget)
 
         repo = repoagent.rawRepo()
-        ctx = repo[rev]
+        ctx = scmutil.revsymbol(repo, rev)
         form.addRow(_('Revision:'), QLabel('%d (%s)' % (ctx.rev(), ctx)))
         self.rev = ctx.rev()
 
@@ -51,6 +77,7 @@ class TagDialog(QDialog):
         self.tagCombo = QComboBox()
         self.tagCombo.setEditable(True)
         self.tagCombo.setEditText(hglib.tounicode(tag))
+        self.tagCombo.setMinimumContentsLength(30)  # cut long name
         self.tagCombo.currentIndexChanged.connect(self.updateStates)
         self.tagCombo.editTextChanged.connect(self.updateStates)
         qtlib.allowCaseChangingInput(self.tagCombo)
@@ -62,11 +89,9 @@ class TagDialog(QDialog):
         ### options
         expander = qtlib.ExpanderLabel(_('Options'), False)
         expander.expanded.connect(self.show_options)
-        box.addWidget(expander)
-
         optbox = QVBoxLayout()
         optbox.setSpacing(6)
-        box.addLayout(optbox)
+        form.addRow(expander, optbox)
 
         hbox = QHBoxLayout()
         hbox.setSpacing(0)
@@ -96,7 +121,7 @@ class TagDialog(QDialog):
         bbox.rejected.connect(self.reject)
         self.addBtn = bbox.addButton(_('&Add'), BB.ActionRole)
         self.removeBtn = bbox.addButton(_('&Remove'), BB.ActionRole)
-        box.addWidget(bbox)
+        form.addRow(bbox)
 
         self.addBtn.clicked.connect(self.onAddTag)
         self.removeBtn.clicked.connect(self.onRemoveTag)
@@ -209,8 +234,8 @@ class TagDialog(QDialog):
         self.customTextLineEdit.setVisible(visible)
 
     def set_status(self, text, icon):
-        self.status.setShown(True)
-        self.sep.setShown(True)
+        self.status.setVisible(True)
+        self.sep.setVisible(True)
         self.status.set_status(text, icon)
 
     def clear_status(self):
