@@ -21,19 +21,27 @@ from mercurial import (
     encoding,
     error,
     extensions,
+    fancyopts,
     filemerge,
-    fileset,
+    filesetlang,
     mdiff,
     merge as mergemod,
+    patch as patchmod,
     pathutil,
     rcutil,
-    registrar,
     revset as revsetmod,
     revsetlang,
-    templatefilters,
+    scmutil,
+    state as statemod,
+    subrepoutil,
     ui as uimod,
     util,
 )
+from mercurial.utils import (
+    dateutil,
+    stringutil,
+)
+
 from tortoisehg.util import paths
 
 from mercurial.node import nullrev
@@ -43,6 +51,7 @@ from tortoisehg.util.i18n import (
     ngettext as _ngettext,
 )
 
+nullsubrepostate = subrepoutil.nullstate
 _encoding = encoding.encoding
 _fallbackencoding = encoding.fallbackencoding
 
@@ -55,13 +64,7 @@ _extensions_blacklist = (
     'zeroconf',
 )
 
-try:
-    command = registrar.command
-except AttributeError:
-    # hg<4.3 (d47d7d3bd07b)
-    from mercurial import cmdutil
-    command = cmdutil.command
-
+extractpatch = patchmod.extract
 tokenizerevspec = revsetlang.tokenize
 userrcpath = rcutil.userrcpath
 
@@ -197,7 +200,7 @@ def getrevisionlabel(repo, rev):
 
     ctx = repo[rev]
     label = _getfirstrevisionlabel(repo, ctx)
-    if label and ctx == repo[label]:
+    if label and ctx == scmutil.revsymbol(repo, label):
         return label
 
     return str(rev)
@@ -219,6 +222,23 @@ def getcurrentqqueue(repo):
         cur = cur[8:]
     return cur
 
+def gitcommit(ctx):
+    """
+    If the hggit extension is loaded, and the repository is a git clone,
+    returns the git commit hash of the current revision
+    """
+
+    repo = ctx._repo
+
+    if 'hggit' not in repo.extensions():
+        return None
+
+    fullgitnode = repo.githandler.map_git_get(ctx.hex())
+    if fullgitnode is None:
+        return None
+
+    return fullgitnode[:12]
+
 def getqqueues(repo):
     ui = repo.ui.copy()
     ui.quiet = True  # don't append "(active)"
@@ -227,10 +247,26 @@ def getqqueues(repo):
         opts = {'list': True}
         mqmod.qqueue(ui, repo, None, **opts)
         qqueues = tounicode(ui.popbuffer()).splitlines()
-    except (util.Abort, EnvironmentError):
+    except (error.Abort, EnvironmentError):
         qqueues = []
     return qqueues
 
+def readgraftstate(repo):
+    """Read a list of nodes from graftstate; or None if nothing in progress"""
+    graftstatefile = repo.vfs.join('graftstate')
+    if not os.path.exists(graftstatefile):
+        return
+    with open(graftstatefile, 'r') as f:
+        info = f.readlines()
+    if info and info[0] == '1\n':
+        # doesn't look like a list of nodes
+        return statemod.cmdstate(repo, 'graftstate').read()['nodes']
+    if len(info):
+        revlist = [rev.strip() for rev in info]
+        revlist = [rev for rev in revlist if rev != '']
+        if revlist:
+            return revlist
+
 readmergestate = mergemod.mergestate.read
 
 def readundodesc(repo):
@@ -244,7 +280,11 @@ def readundodesc(repo):
     return '', len(repo)
 
 def unidifftext(a, ad, b, bd, fn1, fn2, opts=mdiff.defaultopts):
-    headers, hunks = mdiff.unidiff(a, ad, b, bd, fn1, fn2, opts)
+    binary = stringutil.binary(a) or stringutil.binary(b)
+    headers, hunks = mdiff.unidiff(a, ad, b, bd, fn1, fn2,
+                                   binary=binary, opts=opts)
+    if not hunks:
+        return ''
     text = ''.join(sum((list(hlines) for _hrange, hlines in hunks), []))
     return '\n'.join(headers) + '\n' + text
 
@@ -267,11 +307,13 @@ def allextensions():
     disabledexts = disabledextensions()
     exts = (disabledexts or {}).copy()
     exts.update(enabledexts)
+    exts.pop('configitems')   # tortoisehg.util.configitems
     if hasattr(sys, "frozen"):
         if 'hgsubversion' not in exts:
             exts['hgsubversion'] = _('hgsubversion packaged with thg')
         if 'hggit' not in exts:
             exts['hggit'] = _('hggit packaged with thg')
+        exts.pop('mercurial_extension_utils', None)  # Part of keyring extension
     return exts
 
 def validateextensions(enabledexts):
@@ -293,7 +335,7 @@ def validateextensions(enabledexts):
         exts['perfarce'] = _('perfarce is incompatible with hgsubversion')
     return exts
 
-def _loadextensionwithblacklist(orig, ui, name, path):
+def _loadextensionwithblacklist(orig, ui, name, path, *args, **kwargs):
     if name.startswith('hgext.') or name.startswith('hgext/'):
         shortname = name[6:]
     else:
@@ -301,7 +343,7 @@ def _loadextensionwithblacklist(orig, ui, name, path):
     if shortname in _extensions_blacklist and not path:  # only bundled ext
         return
 
-    return orig(ui, name, path)
+    return orig(ui, name, path, *args, **kwargs)
 
 def _wrapextensionsloader():
     """Wrap extensions.load(ui, name) for blacklist to take effect"""
@@ -311,27 +353,8 @@ def _wrapextensionsloader():
 def loadextensions(ui):
     """Load and setup extensions for GUI process"""
     _wrapextensionsloader()  # enable blacklist of extensions
-    loadedbefore = set(name for name, module in extensions.extensions())
     extensions.loadall(ui)
 
-    # we still generate diffs and commit descriptions in GUI process, which
-    # may require template functions loaded from extensions; so run loaders
-    # of template* tables just like dispatch._dispatch(). (issue #4515)
-    try:
-        extraloaders = dispatchmod.extraloaders
-    except AttributeError:
-        return
-    # hg<4.3 (45b0e9d05ee9)
-    for name, module in extensions.extensions():
-        if name in loadedbefore:
-            continue
-        for objname, loadermod, loadername in extraloaders:
-            if not objname.startswith('template'):
-                continue
-            extraobj = getattr(module, objname, None)
-            if extraobj is not None:
-                getattr(loadermod, loadername)(ui, name, extraobj)
-
 # TODO: provide singular canonpath() wrapper instead?
 def canonpaths(list):
     'Get canonical paths (relative to root) for list of files'
@@ -343,7 +366,7 @@ def canonpaths(list):
     for f in list:
         try:
             canonpats.append(pathutil.canonpath(root, cwd, f))
-        except util.Abort:
+        except error.Abort:
             # Attempt to resolve case folding conflicts.
             fu = f.upper()
             cwdu = cwd.upper()
@@ -564,7 +587,7 @@ def tortoisehgtools(uiorconfig, selectedlocation=None):
         toolname, field = key.split('.', 1)
         if toolname not in tools:
             tools[toolname] = {}
-        bvalue = util.parsebool(value)
+        bvalue = stringutil.parsebool(value)
         if bvalue is not None:
             value = bvalue
         tools[toolname][field] = value
@@ -615,7 +638,7 @@ def copydynamicconfig(srcui, destui):
         destui.setconfig(section, name, value, source)
 
 def shortreponame(ui):
-    name = ui.config('web', 'name')
+    name = ui.config('web', 'name', None)
     if not name:
         return
     src = ui.configsource('web', 'name')  # path:line
@@ -646,7 +669,7 @@ def extractchoices(prompttext):
     return msg, zip(resps, choices)
 
 def displaytime(date):
-    return util.datestr(date, '%Y-%m-%d %H:%M:%S %1%2')
+    return dateutil.datestr(date, '%Y-%m-%d %H:%M:%S %1%2')
 
 def utctime(date):
     return time.strftime("%Y-%m-%d %H:%M:%S", time.gmtime(date[0]))
@@ -674,7 +697,7 @@ def age(date):
     if delta == 0:
         return _('now')
     if delta > agescales[0][1] * 2:
-        return util.shortdate(date)
+        return dateutil.shortdate(date)
 
     for t, s in agescales:
         n = delta // s
@@ -686,7 +709,6 @@ def configuredusername(ui):
     # may fall back to the system default.
     if (not os.environ.get('HGUSER')
         and not ui.config('ui', 'username')
-        and not ui.config('ui', 'user')  # hg<4.3 (e714159860fd)
         and not os.environ.get('EMAIL')):
         return None
     try:
@@ -695,9 +717,9 @@ def configuredusername(ui):
         return None
 
 def username(user):
-    author = templatefilters.person(user)
+    author = stringutil.person(user)
     if not author:
-        author = util.shortuser(user)
+        author = stringutil.shortuser(user)
     return author
 
 def user(ctx):
@@ -716,6 +738,16 @@ def user(ctx):
         user = ''
     return user
 
+def getestimatedsize(fctx):
+    """Return the size of the given fctx without loading the revision text"""
+    if fctx.rev() is None:
+        return fctx.size()
+    else:
+        # fctx.size() can read all data into memory in rename cases so
+        # we read the size directly from the filelog, this is deeper
+        # under the API than I prefer to go, but seems necessary
+        return fctx._filelog._revlog.rawsize(fctx.filerev())
+
 def get_revision_desc(fctx, curpath=None):
     """return the revision description as a string"""
     author = tounicode(username(fctx.user()))
@@ -811,7 +843,10 @@ def parseconfigopts(ui, args):
     >>> u.config('extensions', 'mq')
     '!'
     """
-    config = dispatchmod._earlygetopt(['--config'], args)
+    config = dispatchmod._earlyparseopts(ui, args)['config']
+    # drop --config from args
+    args[:] = fancyopts.earlygetopt(args, '', ['config='],
+                                    gnu=True, keepsep=True)[1]
     return dispatchmod._parseconfig(ui, config)
 
 
@@ -837,10 +872,9 @@ def escapeascii(s):
     r"""Escape string to be embedded as a literal; like Python string_escape,
     but keeps 8bit characters and can process unicode
 
-    >>> from PyQt4.QtCore import QString
     >>> escapeascii("\0 \x0b \x7f \t \n \r ' \\")
     "\\x00 \\x0b \\x7f \\t \\n \\r \\' \\\\"
-    >>> escapeascii(QString(u'\xc0\n'))
+    >>> escapeascii(u'\xc0\n')
     u'\xc0\\n'
     """
     s = _stringify(s)
@@ -850,10 +884,9 @@ def escapepath(path):
     r"""Convert path to command-line-safe string; path must be relative to
     the repository root
 
-    >>> from PyQt4.QtCore import QString
     >>> escapepath('foo/[bar].txt')
     'path:foo/[bar].txt'
-    >>> escapepath(QString(u'\xc0'))
+    >>> escapepath(u'\xc0')
     u'\xc0'
     """
     p = _stringify(path)
@@ -977,7 +1010,7 @@ def formatfilespec(expr, *args):
     but the list must not be empty.
     """
     listfuncs = {}
-    return _formatspec(expr, args, fileset.parse, listfuncs)
+    return _formatspec(expr, args, filesetlang.parse, listfuncs)
 
 def formatrevspec(expr, *args):
     r"""Build revset expression by template and positional arguments
@@ -1009,22 +1042,28 @@ def buildcmdargs(name, *args, **opts):
     r"""Build list of command-line arguments
 
     >>> buildcmdargs('push', branch='foo')
-    ['push', '--branch', 'foo']
+    ['push', '--branch=foo']
     >>> buildcmdargs('graft', r=['0', '1'])
-    ['graft', '-r', '0', '-r', '1']
+    ['graft', '-r0', '-r1']
     >>> buildcmdargs('diff', r=[0, None])
-    ['diff', '-r', '0']
+    ['diff', '-r0']
     >>> buildcmdargs('log', no_merges=True, quiet=False, limit=None)
     ['log', '--no-merges']
     >>> buildcmdargs('commit', user='')
-    ['commit', '--user', '']
+    ['commit', '--user=']
+    >>> buildcmdargs('log', keyword=['', 'foo'])
+    ['log', '--keyword=', '--keyword=foo']
+    >>> buildcmdargs('log', k='')
+    ['log', '-k', '']
+    >>> buildcmdargs('log', k=['', 'foo'])
+    ['log', '-k', '', '-kfoo']
 
     positional arguments:
 
     >>> buildcmdargs('add', 'foo', 'bar')
     ['add', 'foo', 'bar']
     >>> buildcmdargs('cat', '-foo', rev='0')
-    ['cat', '--rev', '0', '--', '-foo']
+    ['cat', '--rev=0', '--', '-foo']
     >>> buildcmdargs('qpush', None)
     ['qpush']
     >>> buildcmdargs('update', '')
@@ -1032,25 +1071,25 @@ def buildcmdargs(name, *args, **opts):
 
     type conversion to string:
 
-    >>> from PyQt4.QtCore import QString
     >>> buildcmdargs('email', r=[0, 1])
-    ['email', '-r', '0', '-r', '1']
+    ['email', '-r0', '-r1']
     >>> buildcmdargs('grep', 'foo', rev=2)
-    ['grep', '--rev', '2', 'foo']
+    ['grep', '--rev=2', 'foo']
     >>> buildcmdargs('tag', u'\xc0', message=u'\xc1')
-    ['tag', '--message', u'\xc1', u'\xc0']
-    >>> buildcmdargs(QString('tag'), QString(u'\xc0'), message=QString(u'\xc1'))
-    [u'tag', '--message', u'\xc1', u'\xc0']
+    ['tag', u'--message=\xc1', u'\xc0']
     """
     fullargs = [_stringify(name)]
     for k, v in opts.iteritems():
         if v is None:
             continue
 
-        if len(k) == 1:
+        ashort = (len(k) == 1)
+        if ashort:
             aname = '-%s' % k
+            apref = aname
         else:
             aname = '--%s' % k.replace('_', '-')
+            apref = aname + '='
         if isinstance(v, bool):
             if v:
                 fullargs.append(aname)
@@ -1058,11 +1097,17 @@ def buildcmdargs(name, *args, **opts):
             for e in v:
                 if e is None:
                     continue
-                fullargs.append(aname)
-                fullargs.append(_stringify(e))
+                s = _stringify(e)
+                if s or not ashort:
+                    fullargs.append(apref + s)
+                else:
+                    fullargs.extend([aname, s])
         else:
-            fullargs.append(aname)
-            fullargs.append(_stringify(v))
+            s = _stringify(v)
+            if s or not ashort:
+                fullargs.append(apref + s)
+            else:
+                fullargs.extend([aname, s])
 
     args = [_stringify(v) for v in args if v is not None]
     if any(e.startswith('-') for e in args):
